@@ -17,7 +17,7 @@
 
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
 #ifndef CAIROFACEINFO_H
@@ -32,6 +32,7 @@
 
 	cairo_font_slant_t _c_slant;
 	cairo_font_weight_t _c_weight;
+	cairo_font_face_t *_fontFace; 
 
 	NSString *_faceName;
 	NSString *_familyName;
@@ -67,6 +68,7 @@
 - (void) setName: (NSString *)name;
 - (const char *) cairoCName;
 - (void) setCairoName: (NSString *)name;
+- (cairo_font_face_t *)fontFace;
 
 
 
