@@ -26,27 +26,19 @@
 #include <cairo.h>
 #include <Foundation/Foundation.h>
 
-typedef struct _CairoInfo
-{
-} CairoInfo;
-
 @interface CairoSurface : NSObject
 {
 @public
   void *gsDevice;
+  cairo_surface_t *_surface;
 }
 
-+ (CairoSurface *) surfaceForDevice: (void *) device
-                        depthInfo: (CairoInfo *) cairoInfo;
-
-+ (CairoSurface *) createSurfaceForDevice:(void *)device
-								depthInfo:(CairoInfo *)cairoInfo;
-
-- (id) initWithDevice:(void *)device;
-- (void) setAsTargetOfCairo:(cairo_t *)ct;
+- (id) initWithDevice: (void *)device;
 
 - (NSSize) size;
 
+- (cairo_surface_t *) surface;
+
 @end
 
 #endif
