@@ -16,16 +16,17 @@
 
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 #include "cairo/CairoContext.h"
 #include "cairo/CairoGState.h"
+#include "cairo/CairoFontInfo.h"
+#include "cairo/CairoFontEnumerator.h"
 #include "NSBezierPathCairo.h"
+#include "x11/XGServer.h"
 
 #define XRGSTATE ((CairoGState *)gstate)
 
-@class CairoFontEnumerator;
-@class CairoFontInfo;
 
 @implementation CairoContext
 
@@ -35,18 +36,11 @@
   [NSGraphicsContext setDefaultContextClass:self];
 
   [CairoSurface initializeBackend];
-  [CairoFontEnumerator initializeBackend];
-  [CairoFontInfo initializeBackend];
+  [GSFontEnumerator setDefaultClass: [CairoFontEnumerator class]];
+  [GSFontInfo setDefaultClass: [CairoFontInfo class]];
   [NSBezierPath initializeCairoBezierPath];
 }
 
-/* FIXME: TRAP */
-- (void) forwardInvocation: (NSInvocation *)anInvocation
-{
-  /* only for trapping any unknown message. */
-  NSLog (@":::UNKNOWN::: %@ %@", self, anInvocation);
-}
-
 - (void) GSWSetViewIsFlipped: (BOOL)flipped
 {
   if (gstate)
@@ -59,46 +53,17 @@
 {
   NSString *contextType;
 
-  /*
-     CSMKColorType hsb_double;
-     CSMKColorAttributes attr;
-   */
-
   [super initWithContextInfo:info];
 
   contextType = [info objectForKey:
 			 NSGraphicsContextRepresentationFormatAttributeName];
   if (contextType)
     {
-      if (!([contextType isEqualToString: NSGraphicsContextPDFFormat] ||
-	    [contextType isEqualToString: NSGraphicsContextPSFormat]))
-	{
-	  NSLog(@"CairoContext : Hmmmm");
-	}
-      NSLog(@"CairoContext : NSGCRFAName %@", contextType);
+      /* Most likely this is a PS or PDF context, so just return what
+	 super gave us */
       return self;
     }
 
-  /*
-     attr.signature = CSMKHSVColorSignature;
-     attr.isPremultiplied = NO;
-     attr.isPlanar = NO;
-     attr.hasAlphaChannel = NO;
-     attr.isEndian16 = NO;
-     attr.flavor = CSMKBlackIsZeroFlavor;
-     attr.numberOfColorComponents = 3;
-     attr.bytesPerChannel = 4;
-
-     hsb_double = CSMKMakeColorType(&attr);
-
-     ASSIGN(_systemRGBprofile,
-     [CSMKColorProfile defaultColorProfileForColorType:CSMKRGBDoubleColor]);
-     ASSIGN(_systemCMYKprofile,
-     [CSMKColorProfile defaultColorProfileForColorType:CSMKCMYKDoubleColor]);
-     ASSIGN(_systemHSBprofile,
-     [CSMKColorProfile defaultColorProfileForColorType:hsb_double]);
-   */
-
   gstate = [[CairoGState allocWithZone: [self zone]] initWithDrawContext: self];
 
   return self;
@@ -106,13 +71,7 @@
 
 - (void) flushGraphics
 {
-  /* FIXME */
-}
-
-+ (void) handleExposeRect: (NSRect)rect forDriver: (void *)driver
-{
-  NSLog (@"expose");
-  /* FIXME: */
+  XFlush([(XGServer *)server xDisplay]);
 }
 
 @end 
