@@ -17,10 +17,12 @@
 
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
 #include "cairo/CairoFaceInfo.h"
+#include <cairo-ft.h>
+#include <AppKit/NSFontManager.h>
 
 @ implementation CairoFaceInfo 
 
@@ -45,6 +47,19 @@
   return self;
 }
 
+- (void) dealloc
+{
+  if (_fontFace)
+    {
+      cairo_font_face_destroy(_fontFace);
+    }
+  RELEASE(_familyName);
+  RELEASE(_faceName);
+  RELEASE(_displayName);
+  RELEASE(_cairoName);
+  [super dealloc];
+}
+
 - (void) setFamilyName: (NSString *)name
 {
   ASSIGN(_familyName, name);
@@ -130,4 +145,21 @@
   return 257;
 }
 
+extern cairo_font_face_t *
+_cairo_toy_font_face_create(const char          *family, 
+			    cairo_font_slant_t   slant, 
+			    cairo_font_weight_t  weight);
+
+- (cairo_font_face_t *)fontFace
+{
+  if (!_fontFace)
+    {
+      // FIXME: This function is not exported by cairo
+      _fontFace = _cairo_toy_font_face_create([_cairoName cString], 
+					      _c_slant, _c_weight);
+    }
+
+  return _fontFace;
+}
+
 @end
