@@ -17,7 +17,7 @@
 
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
 #include <Foundation/NSObject.h>
@@ -36,12 +36,22 @@
 
 #include "gsc/GSGState.h"
 #include "cairo/CairoFontEnumerator.h"
-#include "cairo/CairoFontManager.h"
+#include "cairo/CairoFontInfo.h"
 
 @implementation CairoFontEnumerator 
 
-+ (void) initializeBackend
+NSMutableDictionary * __allFonts;
+
++ (CairoFaceInfo *) fontWithName: (NSString *) name
 {
+  CairoFaceInfo *face;
+
+  face =[__allFonts objectForKey: name];
+  if (!face)
+    {
+      NSLog (@"Font not found %@", name);
+    }
+  return face;
 }
 
 - (void) enumerateFontsAndFamilies
@@ -52,29 +62,61 @@
     {
       NSArray *fontDef;
       NSMutableArray *fontDefs;
+      CairoFaceInfo *aFace;
 
-      ASSIGN(allFontNames, [CairoFontManager allFontNames]);
+      __allFonts = [[NSMutableDictionary alloc] init];
       allFontFamilies =[[NSMutableDictionary alloc] init];
 
       fontDefs =[NSMutableArray arrayWithCapacity:10];
       [allFontFamilies setObject: fontDefs forKey:@"Helvetica"];
+
       fontDef =[NSArray arrayWithObjects: @"Helvetica", @"Medium",
 			[NSNumber numberWithInt: 5],
 			[NSNumber numberWithUnsignedInt:0], nil];
       [fontDefs addObject:fontDef];
-      
+      aFace = [[CairoFaceInfo alloc] initWithName: @"Medium" 
+				     familyName: @"Helvetica" 
+				     displayName: @"Helvetica" 
+				     cairoName: @"serif" 
+				     weight: 5 
+				     traits: 0 
+				     cairoSlant: CAIRO_FONT_SLANT_NORMAL 
+				     cairoWeight: CAIRO_FONT_WEIGHT_NORMAL];
+      [__allFonts setObject: aFace forKey: @"Helvetica"];
+      RELEASE(aFace);
+    
       fontDef =[NSArray arrayWithObjects: @"Helvetica-Bold", @"Bold",
 			[NSNumber numberWithInt: 9],
 			[NSNumber numberWithUnsignedInt:NSBoldFontMask],
 			nil];
       [fontDefs addObject:fontDef];
+      aFace = [[CairoFaceInfo alloc] initWithName: @"Bold" 
+				     familyName: @"Helvetica" 
+				     displayName: @"Helvetica Bold" 
+				     cairoName: @"serif" 
+				     weight: 9 
+				     traits: NSBoldFontMask 
+				     cairoSlant: CAIRO_FONT_SLANT_NORMAL 
+				     cairoWeight: CAIRO_FONT_WEIGHT_BOLD];
+      [__allFonts setObject: aFace forKey: @"Helvetica-Bold"];
+      RELEASE(aFace);
       
       fontDef =[NSArray arrayWithObjects: @"Helvetica-Oblique", @"Oblique",
 			[NSNumber numberWithInt: 5],
 			[NSNumber numberWithUnsignedInt:NSItalicFontMask],
 			nil];
       [fontDefs addObject:fontDef];
-      
+      aFace = [[CairoFaceInfo alloc] initWithName: @"Oblique" 
+				     familyName: @"Helvetica" 
+				     displayName: @"Helvetica Oblique" 
+				     cairoName: @"serif" 
+				     weight: 5 
+				     traits: NSItalicFontMask 
+				     cairoSlant: CAIRO_FONT_SLANT_OBLIQUE 
+				     cairoWeight: CAIRO_FONT_WEIGHT_NORMAL];
+      [__allFonts setObject: aFace forKey: @"Helvetica-Oblique"];
+      RELEASE(aFace);
+
       fontDefs =[NSMutableArray arrayWithCapacity:10];
       [allFontFamilies setObject: fontDefs forKey:@"Courier"];
       
@@ -83,10 +125,21 @@
 			[NSNumber numberWithUnsignedInt:NSFixedPitchFontMask],
 			nil];
       [fontDefs addObject:fontDef];
+      aFace = [[CairoFaceInfo alloc] initWithName: @"Medium" 
+				     familyName: @"Courier" 
+				     displayName: @"Courier" 
+				     cairoName: @"Courier" 
+				     weight: 5 
+				     traits: NSFixedPitchFontMask 
+				     cairoSlant: CAIRO_FONT_SLANT_NORMAL 
+				     cairoWeight: CAIRO_FONT_WEIGHT_NORMAL];
+      [__allFonts setObject: aFace forKey: @"Courier"];
+      RELEASE(aFace);
       
+      ASSIGN(allFontNames, [__allFonts allKeys]);
       done = YES;
     }
-  NSLog (@"%@", allFontNames);
+  //NSLog (@"%@", allFontNames);
 }
 
 - (NSString *) defaultSystemFontName
