@@ -17,26 +17,18 @@
 
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
 #include <AppKit/NSAffineTransform.h>
 #include "cairo/CairoFontInfo.h"
 #include "cairo/CairoFontEnumerator.h"
-#include "cairo/CairoFontManager.h"
 
 #include <math.h>
-#include <cairo.h>
+#include <cairo-ft.h>
 
 @implementation CairoFontInfo 
 
-+ (void) initializeBackend
-{
-  //NSLog(@"CairoFontInfo : Initializing...");
-  [GSFontEnumerator setDefaultClass: [CairoFontEnumerator class]];
-  [GSFontInfo setDefaultClass: self];
-}
-
 - (void) setCacheSize: (unsigned int)size
 {
   _cacheSize = size;
@@ -54,13 +46,16 @@
 
 - (BOOL) setupAttributes
 {
-  cairo_matrix_t *font_matrix;
   cairo_font_extents_t font_extents;
+  cairo_font_face_t *face;
+  cairo_matrix_t font_matrix;
+  cairo_matrix_t ctm;
+  cairo_font_options_t *options;
 
   /* do not forget to check for font specific
    * cache size from face info FIXME FIXME
    */
-  ASSIGN(_faceInfo, [CairoFontManager fontWithName: fontName]);
+  ASSIGN(_faceInfo, [CairoFontEnumerator fontWithName: fontName]);
   if (!_faceInfo)
     {
       return NO;
@@ -83,19 +78,24 @@
   /* setting GSFontInfo:
    * xHeight, pix_width, pix_height
    */
-  _cf = cairo_create();
-  cairo_select_font(_cf, [_faceInfo cairoCName], [_faceInfo cairoSlant],
-		    [_faceInfo cairoWeight]);
-  xrFont = cairo_current_font(_cf);
-
-  font_matrix = cairo_matrix_create();
-  cairo_matrix_set_affine(font_matrix, matrix[0], matrix[1], matrix[2],
-			  -matrix[3], matrix[4], matrix[5]);
-  cairo_transform_font(_cf, font_matrix);
-  cairo_matrix_destroy(font_matrix);
-
-  cairo_current_font_extents(_cf, &font_extents);
-  ascender = font_extents.ascent;
+  cairo_matrix_init(&font_matrix, matrix[0], matrix[1], matrix[2],
+		    -matrix[3], matrix[4], matrix[5]);
+  cairo_matrix_init_identity(&ctm);
+  // FIXME: Should get default font options from somewhere
+  options = cairo_font_options_create();
+  face = [_faceInfo fontFace];
+  if (!face)
+    {
+      return NO;
+    }
+  _scaled = cairo_scaled_font_create(face, &font_matrix, &ctm, options);
+  if (!_scaled)
+    {
+      return NO;
+    }
+  cairo_scaled_font_extents(_scaled, &font_extents);
+  // FIXME: Need some adjustment here
+  ascender = font_extents.ascent + 3;
   descender = font_extents.descent;
   xHeight = font_extents.height;
   maximumAdvancement = NSMakeSize(font_extents.max_x_advance, 
@@ -139,7 +139,10 @@
 - (void) dealloc
 {
   RELEASE(_faceInfo);
-  cairo_destroy(_cf);
+  if (_scaled)
+    {
+      cairo_scaled_font_destroy(_scaled);
+    }
   free(_cachedSizes);
   free(_cachedGlyphs);
   [super dealloc];
@@ -151,13 +154,53 @@
   return YES;
 }
 
+// FIXME: This function is not exported by Cairo. The parameters have changed in recent cairo!!!
+extern
+cairo_status_t
+_cairo_scaled_font_text_to_glyphs (cairo_scaled_font_t *scaled_font,
+				   const char          *utf8, 
+				   cairo_glyph_t      **glyphs, 
+				   int 		       *num_glyphs);
+
+static
+BOOL _cairo_extents_for_text(cairo_scaled_font_t *scaled_font, const char *utf8,
+			     cairo_text_extents_t *ctext)
+{
+  cairo_glyph_t *glyphs = NULL;
+  int num_glyphs;
+  cairo_status_t status;
+
+  status = _cairo_scaled_font_text_to_glyphs(scaled_font, utf8, 
+					     &glyphs, &num_glyphs);
+  if (glyphs)
+    {
+      cairo_scaled_font_glyph_extents(scaled_font, glyphs, num_glyphs, ctext);
+      free(glyphs);
+
+      return YES;
+    }
+
+  return NO;
+}
+
+static
+BOOL _cairo_extents_for_NSGlyph(cairo_scaled_font_t *scaled_font, NSGlyph glyph,
+				cairo_text_extents_t *ctext)
+{
+  char str[3];
+
+  // FIXME: This is wrong for none ASCII characters!
+  str[0] = glyph;
+  str[1] = 0;
+  
+  return _cairo_extents_for_text(scaled_font, str, ctext);
+}
+
 - (NSSize) advancementForGlyph: (NSGlyph)glyph
 {
-  cairo_glyph_t cglyph;
   cairo_text_extents_t ctext;
   int entry;
 
-  glyph -= 29;
   entry = glyph % _cacheSize;
 
   if (_cachedGlyphs[entry] == glyph)
@@ -165,38 +208,40 @@
       return _cachedSizes[entry];
     }
 
-  cglyph.index = glyph;
-  cglyph.x = 0;
-  cglyph.y = 0;
-  cairo_glyph_extents(_cf, &cglyph, 1, &ctext);
-  _cachedGlyphs[entry] = glyph;
-  _cachedSizes[entry] = NSMakeSize(ctext.x_advance, ctext.y_advance);
+  if (_cairo_extents_for_NSGlyph(_scaled, glyph, &ctext))
+    {
+      _cachedGlyphs[entry] = glyph;
+      _cachedSizes[entry] = NSMakeSize(ctext.x_advance, ctext.y_advance);
 
-  return _cachedSizes[entry];
+      return _cachedSizes[entry];
+    }
+
+  return NSZeroSize;
 }
 
 - (NSRect) boundingRectForGlyph: (NSGlyph)glyph
 {
-  cairo_glyph_t cglyph;
   cairo_text_extents_t ctext;
 
-  glyph -= 29;
-  cglyph.index = glyph;
-  cglyph.x = 0;
-  cglyph.y = 0;
-  cairo_glyph_extents(_cf, &cglyph, 1, &ctext);
+  if (_cairo_extents_for_NSGlyph(_scaled, glyph, &ctext))
+    {
+      return NSMakeRect(ctext.x_bearing, ctext.y_bearing,
+			ctext.width, ctext.height);
+    }
 
-  return NSMakeRect(ctext.x_bearing, ctext.y_bearing,
-		    ctext.width, ctext.height);
+  return NSZeroRect;
 }
 
 - (float) widthOfString: (NSString *)string
 {
   cairo_text_extents_t ctext;
 
-  cairo_text_extents(_cf, [string UTF8String], &ctext);
+  if (_cairo_extents_for_text(_scaled, [string UTF8String], &ctext))
+    {
+      return ctext.width;
+    }
 
-  return ctext.width;
+  return 0.0;
 }
 
 -(NSGlyph) glyphWithName: (NSString *) glyphName
@@ -225,89 +270,30 @@
 
      cairo_destroy(ct);
    */
-
-#if 0
-  int i;
-  NSGlyph glyph;
-
-  FT_Matrix ftmatrix;
-  FT_Vector ftdelta;
-
-  NSPoint p = [path currentPoint];
-
-  ftmatrix.xx = 65536;
-  ftmatrix.xy = 0;
-  ftmatrix.yx = 0;
-  ftmatrix.yy = 65536;
-  ftdelta.x = p.x * 64.0;
-  ftdelta.y = p.y * 64.0;
-
-  for (i = 0; i < count; i++, glyphs++)
-    {
-      FT_Face face;
-      FT_Glyph gl;
-      FT_OutlineGlyph og;
-
-      glyph = *glyphs - 1;
-
-      if (FTC_Manager_Lookup_Size(ftc_manager, &_imgd.font, &face, 0))
-	continue;
-      if (FT_Load_Glyph(face, glyph, FT_LOAD_DEFAULT))
-	continue;
-
-      if (FT_Get_Glyph(face->glyph, &gl))
-	continue;
-
-      if (FT_Glyph_Transform(gl, &ftmatrix, &ftdelta))
-	{
-	  NSLog(@"glyph transformation failed!");
-	  continue;
-	}
-      og = (FT_OutlineGlyph)gl;
-
-      ftdelta.x += gl->advance.x >> 10;
-      ftdelta.y += gl->advance.y >> 10;
-
-      FT_Outline_Decompose(&og->outline, &bezierpath_funcs, path);
-      FT_Done_Glyph(gl);
-    }
-
-  if (count)
-    {
-      [path moveToPoint: NSMakePoint(ftdelta.x / 64.0,
-				     ftdelta.y / 64.0)];
-    }
-#endif
 }
 
 - (void) drawGlyphs: (const NSGlyph*)glyphs
 	     length: (int)length 
 	         on: (cairo_t*)ct
-		atX: (double)dx
-		  y: (double)dy
 {
-  static cairo_glyph_t *cglyphs = NULL;
-  static int maxlength = 0;
-  size_t i;
-  cairo_text_extents_t gext;
-
-  if (length > maxlength)
-    {
-      maxlength = length;
-      cglyphs = realloc(cglyphs, sizeof(cairo_glyph_t) * maxlength);
-    }
+  cairo_matrix_t font_matrix;
+  char str[length+1];
+  int i;
 
   for (i = 0; i < length; i++)
     {
-      cglyphs[i].index = glyphs[i] + -29;	/* experimental */
-      cglyphs[i].x = dx;
-      cglyphs[i].y = dy;
-      cairo_glyph_extents(ct, cglyphs + i, 1, &gext);
-      dx += gext.x_advance;
-      dy += gext.y_advance;
+      str[i] = glyphs[i];
     }
+  str[length] = 0;
 
-  cairo_show_glyphs(ct, cglyphs, length);
+  cairo_matrix_init(&font_matrix, matrix[0], matrix[1], matrix[2],
+		    -matrix[3], matrix[4], matrix[5]);
+  cairo_set_font_face(ct, [_faceInfo fontFace]);
+  cairo_set_font_matrix(ct, &font_matrix);
+
+  // FIXME: Need some adjustment here
+  cairo_rel_move_to(ct, 0.0, -5.0);
+  cairo_show_text(ct, str);
 }
  
 @end
