@@ -20,40 +20,31 @@
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#include <Foundation/NSUserDefaults.h>
 #include <math.h>
 #include "cairo/XGCairoSurface.h"
+#include <cairo-xlib.h>
 
 #define GSWINDEVICE ((gswindow_device_t *)gsDevice)
 
 @implementation XGCairoSurface
 
-+ (CairoSurface *) createSurfaceForDevice: (void *)device
-				depthInfo: (CairoInfo *)cairoInfo
-{
-#define NEWGSWINDEVICE ((gswindow_device_t *)device)
-  XGCairoSurface *surface;
-    
-  surface = [[self alloc] initWithDevice: NEWGSWINDEVICE];
-  
-  NSAssert(NEWGSWINDEVICE->buffer, @"FIXME! CairoSurface: Strange, a window doesn't have buffer");
-
-  return surface;
-#undef NEWGSWINDEVICE
-}
-
-
-- (NSString *) description
-{
-  return [NSString stringWithFormat: @"<XGCairoSurface %p xr:%p>", self, xrSurface];
-}
-
 - (id) initWithDevice: (void *)device
 {
-  /* FIXME format is ignore when Visual isn't NULL
-   * Cairo may change this API
-   */
+  Display *dpy;
+  Drawable drawable;
+
   gsDevice = device;
+
+  dpy = GSWINDEVICE->display;
+  if (GSWINDEVICE->type != NSBackingStoreNonretained)
+    {
+      drawable = GSWINDEVICE->buffer;
+    }
+  else
+    {
+      drawable = GSWINDEVICE->ident;
+    }
+
   /*
     if (GSWINDEVICE->type != NSBackingStoreNonretained)
     {
@@ -64,47 +55,17 @@
     }
   */
   
-  xrSurface = cairo_xlib_surface_create(GSWINDEVICE->display,
-					GSWINDEVICE->buffer,
-					DefaultVisual(GSWINDEVICE->display,
-						      DefaultScreen(GSWINDEVICE->display)),
-					0,
-					DefaultColormap(GSWINDEVICE->display,
-							DefaultScreen(GSWINDEVICE->display)));
-
+  _surface = cairo_xlib_surface_create(dpy,
+				       drawable,
+				       DefaultVisual(dpy, DefaultScreen(dpy)),
+				       GSWINDEVICE->xframe.size.width,
+				       GSWINDEVICE->xframe.size.height);
   return self;
 }
 
-- (void) setAsTargetOfCairo: (cairo_t *)ct
-{
-  /*
-    if (GSWINDEVICE->type != NSBackingStoreNonretained)
-    {
-    GSWINDEVICE->gdriverProtocol |= GDriverHandlesExpose;
-    XSetWindowBackgroundPixmap(GSWINDEVICE->display,
-    GSWINDEVICE->ident,
-    GSWINDEVICE->buffer);
-    }
-  */
-  //cairo_set_target_drawable(ct, GSWINDEVICE->display, GSWINDEVICE->buffer);
-  cairo_set_target_surface(ct, xrSurface);
-}
-
-- (void) dealloc
-{
-  cairo_surface_destroy(xrSurface);
-  [super dealloc];
-}
-
-- (void) logDevice
-{
-  NSLog(@"device %p id:%p buff:%p",self,GSWINDEVICE->ident,GSWINDEVICE->buffer);
-}
-
 - (NSSize) size
 {
   return GSWINDEVICE->xframe.size;
 }
 
 @end
-
