@@ -63,23 +63,23 @@
 #define CHECK_NULL_OUTPUT(outvar) \
   do { if (outvar == NULL) {\
     DPS_ERROR(DPSnulloutput, @"NULL output variable specified"); \
-    return; } } while(0)
+    return; } } while (0)
 
 #define CHECK_INVALID_FONT(ident) \
   do { if (ident >= [fontid count]) { \
     DPS_ERROR(DPSinvalidfont, @"Cannot find indicated font"); \
-    return; } } while(0)
+    return; } } while (0)
 
 #define CHECK_STACK_UNDERFLOW(stack) \
   do { if (GSIArrayCount((GSIArray)stack) == 0) { \
     DPS_ERROR(DPSstackunderflow, @"Attempt to pop from empty stack"); \
-    return; } } while(0)
+    return; } } while (0)
 
 #if 0
 #define CHECK_TYPECHECK(obj, kind) \
   do { if ([kind class] != Nil && !GSObjCIsKindOf(GSObjCClass(obj), [kind class])) {\
     DPS_ERROR(DPStypecheck, @"Invalid object"); \
-    return; } } while(0)
+    return; } } while (0)
 #else
 #define CHECK_TYPECHECK(obj,kind)
 #endif
@@ -94,7 +94,7 @@
   } while (0)
 
 #define ctxt_push(object, stack) \
-  GSIArrayAddItem((GSIArray)stack, (GSIArrayItem)object)
+  GSIArrayAddItem((GSIArray)stack, (GSIArrayItem)((id)object))
 
 /* Globally unique gstate number */
 static unsigned int unique_index = 0;
@@ -255,6 +255,11 @@ static unsigned int unique_index = 0;
   [gstate DPSsetrgbcolor:r :g :b];
 }
 
+- (void) GSSetPatterColor: (NSImage*)image 
+{
+  [gstate GSSetPatterColor: image];
+}
+
 - (void) GSSetFillColorspace: (void *)spaceref
 {
   [gstate GSSetFillColorspace: spaceref];
@@ -409,7 +414,7 @@ static unsigned int unique_index = 0;
 
 - (int) GSDefineGState
 {
-  if(gstate == nil)
+  if (gstate == nil)
     {
       DPS_ERROR(DPSundefined, @"No gstate");
       return 0;
@@ -425,7 +430,7 @@ static unsigned int unique_index = 0;
 
 - (void) GSReplaceGState: (int)gst
 {
-  if(gst <= 0)
+  if (gst <= 0)
     return;
   NSMapInsert(gtable, (void *)gst, AUTORELEASE([gstate copy]));
 }
@@ -757,6 +762,27 @@ static unsigned int unique_index = 0;
   [gstate dissolveGState: g fromRect: rect toPoint: p delta: delta];
 }
 
+- (void) GScomposite: (int)gstateNum
+	     toPoint: (NSPoint)aPoint
+	    fromRect: (NSRect)srcRect
+	   operation: (NSCompositingOperation)op
+	    fraction: (float)delta
+{
+  GSGState *g = gstate;
+
+  if (gstateNum)
+    {
+      [self DPSexecuserobject: gstateNum];
+      ctxt_pop(g, opstack, GSGState);
+    }
+
+  [gstate compositeGState: g
+	         fromRect: srcRect
+                  toPoint: aPoint
+                       op: op
+                 fraction: delta];
+}
+
 - (void) GSDrawImage: (NSRect) rect: (void *) imageref
 {
   [self notImplemented: _cmd];
@@ -819,7 +845,7 @@ static unsigned int unique_index = 0;
 
 - (void) GSWSetViewIsFlipped: (BOOL) flipped
 {
-  if(gstate)
+  if (gstate)
     gstate->viewIsFlipped = flipped;
 }
 
