@@ -20,7 +20,7 @@
    
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
    */
 
 #include "config.h"
@@ -171,6 +171,10 @@ fpfloat(FILE *stream, float f)
   [self notImplemented: _cmd];
 }
 
+- (void) GSSetPatterColor: (NSImage*)image 
+{
+  [self notImplemented: _cmd];
+}
 
 /* ----------------------------------------------------------------------- */
 /* Text operations */
@@ -772,6 +776,15 @@ fpfloat(FILE *stream, float f)
   NSLog(@"DPSinvalidcontext: dissolve in a stream context");
 }
 
+- (void) GScomposite: (int)gstateNum
+	     toPoint: (NSPoint)aPoint
+	    fromRect: (NSRect)srcRect
+	   operation: (NSCompositingOperation)op
+	    fraction: (float)delta
+{
+ [self DPScomposite: NSMinX(srcRect) : NSMinY(srcRect) : NSWidth(srcRect) : NSHeight(srcRect) 
+                   : gstateNum : aPoint.x : aPoint.y : op];
+}
 
 - (void) GSDrawImage: (NSRect)rect : (void *)imageref
 {
@@ -850,16 +863,16 @@ static const char *hexdigits = "0123456789abcdef";
 	    bytesPerRow, pixelsHigh, bytes);
       return;
     }
-  if(hasAlpha)
+  if (hasAlpha)
     spp = samplesPerPixel - 1;
   else
     spp = samplesPerPixel;
 
-  if(samplesPerPixel > 1) 
+  if (samplesPerPixel > 1) 
     {
-      if(isPlanar || hasAlpha) 
+      if (isPlanar || hasAlpha) 
 	{
-	  if(bitsPerSample != 8) 
+	  if (bitsPerSample != 8) 
 	    {
 	      NSLog(@"Image format conversion not supported for bps!=8");
 	      return;
@@ -881,7 +894,7 @@ static const char *hexdigits = "0123456789abcdef";
     }
   
   // The context is now waiting for data on its standard input
-  if(isPlanar || hasAlpha) 
+  if (isPlanar || hasAlpha) 
     {
       // We need to do a format conversion.
       // We do this on the fly, sending data to the context as soon as
