@@ -19,22 +19,23 @@
    
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
    */
 
+// Currently the use of alpha blending is switched off by default.
+#define USE_ALPHABLEND
+
+// Define this so we pick up AlphaBlend, when loading windows.h
+#ifdef USE_ALPHABLEND
+#define WINVER 0x0500
+#endif
+
 #include <AppKit/NSAffineTransform.h>
 #include <AppKit/NSBezierPath.h>
 #include <AppKit/NSColor.h>
 #include <AppKit/NSFont.h>
 #include <AppKit/NSGraphics.h>
 
-// Currently the use of alpha blending is switched off by default.
-//#define USE_ALPHABLEND
-
-// Define this so we pick up AlphaBlend, when loading windows.h
-#ifdef USE_ALPHABLEND
-#define WINVER 0x0500
-#endif
 #include "winlib/WIN32GState.h"
 #include "winlib/WIN32Context.h"
 #include "winlib/WIN32FontInfo.h"
@@ -199,11 +200,11 @@ RECT GSXWindowRectToMS(WIN32GState *s, NSRect r)
   return r1;
 }
 
-- (void) _compositeGState: (WIN32GState *) source
-                 fromRect: (NSRect) sourceRect
-                  toPoint: (NSPoint) destPoint
-                       op: (NSCompositingOperation) op
-                 fraction: (float)delta
+- (void) compositeGState: (WIN32GState *) source
+		fromRect: (NSRect) sourceRect
+		 toPoint: (NSPoint) destPoint
+		      op: (NSCompositingOperation) op
+		fraction: (float)delta
 {
   HDC sourceDC;
   HDC hDC;
@@ -224,7 +225,7 @@ RECT GSXWindowRectToMS(WIN32GState *s, NSRect r)
 
   destRect.size = sourceRect.size;
   destRect.origin = destPoint;
-  rectTo = GSViewRectToWin( self, destRect );
+  rectTo = GSViewRectToWin(self, destRect);
   x = rectTo.left;
   y = rectTo.top;
 
@@ -233,19 +234,19 @@ RECT GSXWindowRectToMS(WIN32GState *s, NSRect r)
 
   sourceDC = [source getHDC];
   
-  if( !sourceDC )
+  if (!sourceDC)
     {
       return;
     }
 
-  if( self == source )
+  if (self == source)
     {
       hDC = sourceDC;
     }
   else
     {
       hDC = [self getHDC];
-      if( !hDC )
+      if (!hDC)
         {
 	  [source releaseHDC: sourceDC];
 	  return;
@@ -265,13 +266,17 @@ RECT GSXWindowRectToMS(WIN32GState *s, NSRect r)
 			     sourceDC,
 			     rectFrom.left, rectFrom.top,
 			     (rectFrom.right - rectFrom.left), h, blendFunc);
-	break;
+	/* There is actually a very real chance this could fail, even on 
+	    computers that supposedly support it. It's not known why it
+	    fails though... */
+	if (success)
+	  break;
 #endif
       }
 
     default:
-      success = BitBlt( hDC, x, y, (rectFrom.right - rectFrom.left), h,
-			sourceDC, rectFrom.left, rectFrom.top, SRCCOPY );
+      success = BitBlt(hDC, x, y, (rectFrom.right - rectFrom.left), h,
+			sourceDC, rectFrom.left, rectFrom.top, SRCCOPY);
       break;
     }
 
@@ -296,11 +301,11 @@ RECT GSXWindowRectToMS(WIN32GState *s, NSRect r)
                  toPoint: (NSPoint)aPoint
                       op: (NSCompositingOperation)op
 {
-  [self _compositeGState: (WIN32GState *) source
-	        fromRect: aRect
-	         toPoint: aPoint
-	              op: op
-	        fraction: 1];
+  [self compositeGState: (WIN32GState *) source
+	       fromRect: aRect
+	        toPoint: aPoint
+	             op: op
+	       fraction: 1];
 }
 
 - (void) dissolveGState: (GSGState *)source
@@ -308,11 +313,11 @@ RECT GSXWindowRectToMS(WIN32GState *s, NSRect r)
 		toPoint: (NSPoint)aPoint 
 		  delta: (float)delta
 {
-  [self _compositeGState: (WIN32GState *) source
-	        fromRect: aRect
-	         toPoint: aPoint
-	              op: NSCompositeSourceOver
-	        fraction: delta];
+  [self compositeGState: (WIN32GState *) source
+	       fromRect: aRect
+	        toPoint: aPoint
+	             op: NSCompositeSourceOver
+	       fraction: delta];
 }
 
 - (void) compositerect: (NSRect)aRect
@@ -374,13 +379,15 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
 		       NSString *colorSpaceName,
 		       const unsigned char *const data[5])
 {
+  const unsigned char *bits = data[0];
   HBITMAP hbitmap;
   BITMAPINFO *bitmap;
   BITMAPINFOHEADER *bmih;
   int xres, yres;
   UINT fuColorUse;
 
-  if (isPlanar || ![colorSpaceName isEqualToString: NSDeviceRGBColorSpace])
+  if (isPlanar || !([colorSpaceName isEqualToString: NSDeviceRGBColorSpace] ||
+		    [colorSpaceName isEqualToString: NSCalibratedRGBColorSpace]))
     {
       NSLog(@"Bitmap type currently not supported %d %@", isPlanar, colorSpaceName);
       return NULL;
@@ -399,7 +406,7 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
     bytesPerRow = (bitsPerPixel * pixelsWide) / 8;
 
   // make sure its sane - also handles row padding if hint missing
-  while((bytesPerRow * 8) < (bitsPerPixel * pixelsWide))
+  while ((bytesPerRow * 8) < (bitsPerPixel * pixelsWide))
     bytesPerRow++;
 
   if (!(GetDeviceCaps(hDC, RASTERCAPS) &  RC_DI_BITMAP)) 
@@ -448,15 +455,28 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
     }
   else if (bitsPerPixel == 32)
     {
-      BITMAPV4HEADER *bmih;
+      BITMAPV4HEADER	*bmih;
+      unsigned char	*tmp;
+      unsigned int	pixels = pixelsHigh * pixelsWide;
+      unsigned int	i = 0;
 
       bmih = (BITMAPV4HEADER*)bitmap;
       bmih->bV4Size = sizeof(BITMAPV4HEADER);
       bmih->bV4V4Compression = BI_BITFIELDS;
-      bmih->bV4RedMask = 0x000000FF;
+      bmih->bV4BlueMask = 0x000000FF;
       bmih->bV4GreenMask = 0x0000FF00;
-      bmih->bV4BlueMask = 0x00FF0000;
+      bmih->bV4RedMask = 0x00FF0000;
       bmih->bV4AlphaMask = 0xFF000000;
+      tmp = objc_malloc(pixels * 4);
+      while (i < pixels*4)
+	{
+	  tmp[i+0] = bits[i+2];
+	  tmp[i+1] = bits[i+1];
+	  tmp[i+2] = bits[i+0];
+	  tmp[i+3] = bits[i+3];
+	  i += 4;
+	}
+      bits = tmp;
     }
   else if (bitsPerPixel == 16)
     {
@@ -474,14 +494,17 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
       NSLog(@"Unsure how to handle images with %d bits", bitsPerPixel);
     }
 
-  if (!SetDIBits(hDC, hbitmap, 0, pixelsHigh, data[0], 
-		 bitmap, fuColorUse))
+  if (!SetDIBits(hDC, hbitmap, 0, pixelsHigh, bits, bitmap, fuColorUse))
     {
-      objc_free(bitmap);
       DeleteObject(hbitmap);
-      return NULL;
+      hbitmap = NULL;
     }
 
+  if (bits != data[0])
+    {
+      /* cast bits to Void Pointer to fix warning in compile */
+      objc_free((void *)(bits));
+    }
   objc_free(bitmap);
   return hbitmap;
 }
@@ -614,7 +637,7 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
 
       BeginPath(hDC);
 
-      for(j = 0; j < count; j++) 
+      for (j = 0; j < count; j++) 
         {
 	  type = [path elementAtIndex: j associatedPoints: points];
 	  switch(type) 
@@ -633,9 +656,9 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
 		POINT bp[3];
 		
 #if GDI_WIDELINE_BEZIERPATH_BUG
-		if(drawType == path_stroke && lineWidth > 1)
+		if (drawType == path_stroke && lineWidth > 1)
 		  {
-		    if(j != 0) 
+		    if (j != 0) 
 		      {
 			NSPoint movePoints[3];
 		    
@@ -652,7 +675,7 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
 		  }
 		PolyBezierTo(hDC, bp, 3);
 #if GDI_WIDELINE_BEZIERPATH_BUG
-		if(drawType == path_stroke && lineWidth > 1)
+		if (drawType == path_stroke && lineWidth > 1)
 		  MoveToEx(hDC, bp[2].x, bp[2].y, NULL);
 #endif
 	      }
@@ -875,14 +898,15 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
   [super DPSinitgraphics];
 }
 
-- (void) DPSsetdash: (const float*)pattern : (int)count : (float)phase
+
+- (void) DPSsetdash: (const float*)thePattern : (int)count : (float)phase
 {
   if (!path)
     {
       path = [NSBezierPath new];
     }
 
-  [path setLineDash: pattern count: count phase: phase];
+  [path setLineDash: thePattern count: count phase: phase];
 }
 
 - (void)DPScurrentmiterlimit: (float *)limit 
@@ -991,11 +1015,11 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
   penStyle = PS_GEOMETRIC | PS_SOLID;
   if (path)
     {
-      float pattern[10];
+      float thePattern[10];
       int count = 10;
       float phase;
 
-      [path getLineDash: pattern count: &count phase: &phase];
+      [path getLineDash: thePattern count: &count phase: &phase];
 
       if (count && (count < 10))
 	{
