@@ -893,10 +893,6 @@ NSDebugLLog(@"Frame", @"X2O %d, %@, %@", win->number,
 		      pid_atom, XA_CARDINAL,
 		      32, PropModeReplace, 
 		      (unsigned char*)&pid, 1);
-
-      // We should store the GNUStepMenuImage in the root window
-      // and use that as our title bar icon
-      //  pid_atom = XInternAtom(dpy, "_NET_WM_ICON", False);
     }
 }
 
@@ -956,6 +952,144 @@ NSDebugLLog(@"Frame", @"X2O %d, %@, %@", win->number,
 		 NSHeight(window->xframe));
 }
 
+/*
+ * Code to build up a NET WM icon from our application icon
+ */
+
+-(BOOL) _createNetIcon: (NSImage*)image 
+		result: (long**)pixeldata 
+		  size: (int*)size
+{
+  NSBitmapImageRep *rep;
+  int i, j, w, h, samples;
+  unsigned char *data;
+  int index;
+  long *iconPropertyData;
+  int iconSize;
+ 
+  rep = (NSBitmapImageRep *)[image bestRepresentationForDevice:nil];
+  if (![rep isKindOfClass: [NSBitmapImageRep class]])
+    {
+      NSLog(@"Wrong bitmap class for WM icon %@ %@", rep, image);
+      return NO;
+    }
+
+  if ([rep bitsPerSample] != 8
+      || (![[rep colorSpaceName] isEqual: NSDeviceRGBColorSpace]
+	  && ![[rep colorSpaceName] isEqual: NSCalibratedRGBColorSpace])
+      || [rep isPlanar])
+    {
+      NSLog(@"Wrong image type for WM icon");
+      return NO;
+    }
+
+  h = [rep pixelsHigh];
+  w = [rep pixelsWide];
+  samples = [rep samplesPerPixel];
+  data = [rep bitmapData];
+
+  iconSize = 2 + w * h;
+  iconPropertyData = (long *)objc_malloc(sizeof(long) * iconSize);
+  if (iconPropertyData == NULL)
+    {
+      NSLog(@"No memory for WM icon");
+      return NO;
+    }
+
+  memset(iconPropertyData, 0, sizeof(long)*iconSize);
+  index = 0;
+  iconPropertyData[index++] = w;
+  iconPropertyData[index++] = h;
+
+  for (i = 0; i < h; i++)
+    {
+      unsigned char *d = data;
+
+      for (j = 0; j < w; j++)
+	{
+	  unsigned char A, R, G, B;
+
+	  // red
+	  R = d[0];
+	  // green
+	  G = d[1];
+	  // blue
+	  B = d[2];
+	  // alpha
+#if 0
+/*
+  For unclear reasons the alpha handling does not work, so we simulate it.
+*/
+	  if (samples == 4)
+	    {
+	      A = d[4];
+	    }
+	  else
+	    {
+	      A = 255;
+	    }
+#else
+	  if (R || G || B)
+	    {
+	      A = 255;
+	    }
+	  else
+	    {
+	      A = 0;
+	    }
+#endif
+
+          iconPropertyData[index++] = A << 24 | R << 16 | G << 8 | B;
+	  d += samples;
+	}
+      data += [rep bytesPerRow];
+    }
+  
+  *pixeldata = iconPropertyData;
+  *size = iconSize;
+  return YES;
+}   
+
+- (void) _setNetWMIconFor: (Window) window
+{
+  // We store the GNUstep application icon image in the window
+  // and use that as our title bar icon.
+  // FIXME: This code should rather use the window mini icon, 
+  // but currently this image is not available in the backend.
+  static Atom icon_atom = None;
+  static BOOL didCreateNetIcon = NO;
+  static long *iconPropertyData = NULL;
+  static int iconSize;
+  NSImage *image;
+
+  /* Initialize the atom if needed */
+  if (icon_atom == None)
+    icon_atom = XInternAtom(dpy, "_NET_WM_ICON", False);
+
+  if (!didCreateNetIcon)
+    {
+      if (iconPropertyData != NULL)
+        {
+	  objc_free(iconPropertyData);
+	}
+
+      image = [NSApp applicationIconImage];
+      if (image != nil)
+        {
+	  didCreateNetIcon = YES;
+	  [self _createNetIcon: image result: &iconPropertyData size: &iconSize];
+	}
+    }
+
+  if (iconPropertyData)
+    {
+      XChangeProperty(dpy, window,
+		      icon_atom, XA_CARDINAL, 
+		      32, PropModeReplace,
+		      (unsigned char *)iconPropertyData, iconSize);
+    }
+} 
+
 - (int) window: (NSRect)frame : (NSBackingStoreType)type : (unsigned int)style
 	      : (int)screen
 {
@@ -1071,6 +1205,14 @@ NSDebugLLog(@"Frame", @"X2O %d, %@, %@", win->number,
       setWindowHintsForStyle (dpy, window->ident, style);
     }
 
+  // For window managers supporting EWMH, but not Window Maker, 
+  // where we use a different solution, set the window icon.
+  if (((generic.wm & XGWM_EWMH) != 0) && 
+      ((generic.wm & XGWM_WINDOWMAKER) == 0))
+    {
+      [self _setNetWMIconFor: window->ident];
+    }
+
   // Use the globally active input mode
   window->gen_hints.flags = InputHint;
   window->gen_hints.input = False;
@@ -1346,7 +1488,6 @@ NSDebugLLog(@"Frame", @"X2O %d, %@, %@", win->number,
 
 - (void) titlewindow: (NSString *)window_title : (int) win
 {
-  XTextProperty windowName;
   gswindow_device_t *window;
 
   window = WINDOW_WITH_TAG(win);
@@ -1356,8 +1497,22 @@ NSDebugLLog(@"Frame", @"X2O %d, %@, %@", win->number,
   NSDebugLLog(@"XGTrace", @"DPStitlewindow: %@ : %d", window_title, win);
   if (window_title && window->ident)
     {
-      const char *title = [window_title lossyCString];
-      XStringListToTextProperty((char**)&title, 1, &windowName);
+      XTextProperty windowName;
+      const char *title;
+      int error = XLocaleNotSupported;
+
+#ifdef X_HAVE_UTF8_STRING
+      title = [window_title UTF8String];
+      error = Xutf8TextListToTextProperty(dpy, (char **)&title, 1, 
+					  XUTF8StringStyle,
+					  &windowName);
+#endif
+      if (error != Success) 
+        {
+	  title = [window_title lossyCString];
+	  XStringListToTextProperty((char **)&title, 1,
+				    &windowName);
+	}
       XSetWMName(dpy, window->ident, &windowName);
       XSetWMIconName(dpy, window->ident, &windowName);
       XFree(windowName.value);
@@ -2318,7 +2473,7 @@ static BOOL didCreatePixmaps;
   gui_win = GSWindowWithNumber(win);
 
   n = [window->exposedRects count];
-  if( n > 0 )
+  if (n > 0)
     {
       NSView *v;
       NSValue *val[n];
@@ -2327,7 +2482,7 @@ static BOOL didCreatePixmaps;
       v = [gui_win contentView];
 	
       [window->exposedRects getObjects: val];
-      for( i = 0; i < n; ++i)
+      for (i = 0; i < n; ++i)
 	{
 	  NSRect rect = [val[i] rectValue];
 	  [v setNeedsDisplayInRect: rect];
@@ -3028,7 +3183,8 @@ _computeDepth(int class, int bpp)
   XIconSize *xiconsize;
   int count_return;
   int status;
-  
+  NSSize iconSize;
+
   status = XGetIconSizes(dpy,
                          DefaultRootWindow(dpy),
 			 &xiconsize,
@@ -3039,10 +3195,19 @@ _computeDepth(int class, int bpp)
         {
 	  /* must add 4 pixels for the border which windowmaker leaves out
 	     but gnustep draws over. */
-          return NSMakeSize(xiconsize[0].max_width + 4,
-			    xiconsize[0].max_height + 4);
+          iconSize = NSMakeSize(xiconsize[0].max_width + 4,
+				xiconsize[0].max_height + 4);
 	}
+      else
+        {
+          iconSize = NSMakeSize(xiconsize[0].max_width,
+				xiconsize[0].max_height);
+	}
+
+      XFree(xiconsize);
+      return iconSize;
     }
+
   return [super iconSize];
 }
 
