@@ -1,12 +1,28 @@
-/*
-    XGFontSetFontInfo.m
+/* <title>XGFontSetFontInfo</title>
 
-    NSFont helper for GNUstep X/GPS Backend
+   <abstract>NSFont helper for GNUstep X/GPS Backend</abstract>
 
-    Author: Kazunobu Kuriyama <kazunobu.kuriyama@nifty.com>
-    Date: July 2003
+   Copyright (C) 2003-2005 Free Software Foundation, Inc.
 
- */
+   Author: Kazunobu Kuriyama <kazunobu.kuriyama@nifty.com>
+   Date: July 2003
+   
+   This file is part of the GNU Objective C User Interface library.
+
+   This library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Library General Public
+   License as published by the Free Software Foundation; either
+   version 2 of the License, or (at your option) any later version.
+   
+   This library is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Library General Public License for more details.
+   
+   You should have received a copy of the GNU Library General Public
+   License along with this library; if not, write to the Free
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   */
 
 #include "x11/XGServer.h"
 #include "xlib/XGPrivate.h"
@@ -324,7 +340,7 @@ load_font_set(Display *dpy, const char *given_font_name,
 	      xlfd_elms[11],	// avg width
 	      "*",		// registry
 	      "*"		// encoding
-	     );
+	);
     }
   else
     {
@@ -343,7 +359,7 @@ load_font_set(Display *dpy, const char *given_font_name,
 	      xlfd_elms[10],	// avg width
 	      "*",		// registry
 	      "*"		// encoding
-             );
+);
     }
 
   // N.B. def_string is owned by the X server: Don't release it.
@@ -422,20 +438,20 @@ glyphs2utf8(const NSGlyph* glyphs, int length, UTF8Str* ustr)
       else if (*g < 0x00000800)
 	{
 	  p[i++] = 0xc0 | ((*g >>  6) & 0x1f);
-	  p[i++] = 0x80 | ( *g        & 0x3f);
+	  p[i++] = 0x80 | (*g        & 0x3f);
 	}
       else if (*g < 0x00010000)
 	{
 	  p[i++] = 0xe0 | ((*g >> 12) & 0x0f);
 	  p[i++] = 0x80 | ((*g >>  6) & 0x3f);
-	  p[i++] = 0x80 | ( *g        & 0x3f);
+	  p[i++] = 0x80 | (*g        & 0x3f);
 	}
       else if (*g < 0x00200000)
 	{
 	  p[i++] = 0xf0 | ((*g >> 18) & 0x07);
 	  p[i++] = 0x80 | ((*g >> 12) & 0x3f);
 	  p[i++] = 0x80 | ((*g >>  6) & 0x3f);
-	  p[i++] = 0x80 | ( *g        & 0x3f);
+	  p[i++] = 0x80 | (*g        & 0x3f);
 	}
       else if (*g < 0x04000000)
 	{
@@ -443,7 +459,7 @@ glyphs2utf8(const NSGlyph* glyphs, int length, UTF8Str* ustr)
 	  p[i++] = 0x80 | ((*g >> 18) & 0x3f);
 	  p[i++] = 0x80 | ((*g >> 12) & 0x3f);
 	  p[i++] = 0x80 | ((*g >>  6) & 0x3f);
-	  p[i++] = 0x80 | ( *g        & 0x3f);
+	  p[i++] = 0x80 | (*g        & 0x3f);
 	}
       else if (*g < 0x80000000)
 	{
@@ -452,7 +468,7 @@ glyphs2utf8(const NSGlyph* glyphs, int length, UTF8Str* ustr)
 	  p[i++] = 0x80 | ((*g >> 18) & 0x3f);
 	  p[i++] = 0x80 | ((*g >> 12) & 0x3f);
 	  p[i++] = 0x80 | ((*g >>  6) & 0x3f);
-	  p[i++] = 0x80 | ( *g        & 0x3f);
+	  p[i++] = 0x80 | (*g        & 0x3f);
 	}
       else
 	{
