@@ -19,7 +19,7 @@
    
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
    */
 
 #include "config.h"
@@ -107,12 +107,17 @@ static	Region	emptyRegion;
   alpha_buffer = 0;
   xgcntxt = None;
   agcntxt = None;
+#ifdef HAVE_XFT
+  xft_draw = NULL;
+  xft_alpha_draw = NULL;
+  memset(&xft_color, 0, sizeof(XftColor));
+#endif
   return self;
 }
 
 - (void) dealloc
 {
-  if ( sharedGC == NO && xgcntxt ) 
+  if (sharedGC == NO && xgcntxt) 
     {
       XFreeGC(XDPY, xgcntxt);
     }
@@ -252,6 +257,13 @@ static	Region	emptyRegion;
   gsColorToRGB(&c);
   gcv.foreground = xrRGBToPixel(context, c);
   [self setGCValues: gcv withMask: GCForeground];
+#ifdef HAVE_XFT
+  xft_color.color.red = 65535.0 * c.field[0];
+  xft_color.color.green = 65535.0 * c.field[1];
+  xft_color.color.blue = 65535.0 * c.field[2];
+  xft_color.color.alpha = 0xffff;
+  xft_color.pixel = gcv.foreground;
+#endif
 }
 
 - (void) setAlphaColor: (float)value
@@ -268,6 +280,9 @@ static	Region	emptyRegion;
     agcntxt = XCreateGC(XDPY, draw, GCForeground, &gcv);
   else
     XChangeGC(XDPY, agcntxt, GCForeground, &gcv);
+#ifdef HAVE_XFT
+  xft_color.color.alpha = 65535.0 * value;
+#endif
 }
 
 - (void) copyGraphicContext
@@ -348,6 +363,42 @@ static	Region	emptyRegion;
   return xgcntxt;
 }
 
+#ifdef HAVE_XFT
+- (XftColor) xftColor
+{
+    return xft_color;
+}
+
+- (XftDraw *)xftDrawForDrawable: (Drawable)d
+{
+    if (d == 0)
+        return 0;  //PENDING: warn? throw?
+
+    if (d == draw)
+      {
+        if (xft_draw == NULL)
+            xft_draw =
+                XftDrawCreate(XDPY, d, DefaultVisual(XDPY, DefaultScreen(XDPY)),
+                              DefaultColormap(XDPY, DefaultScreen(XDPY)));
+        if (clipregion != None)
+            XftDrawSetClip(xft_draw, clipregion);
+        return xft_draw;
+      }
+    else if (d == alpha_buffer)
+      {
+        if (xft_alpha_draw == NULL)
+            xft_alpha_draw =
+                XftDrawCreate(XDPY, d, DefaultVisual(XDPY, DefaultScreen(XDPY)),
+                              DefaultColormap(XDPY, DefaultScreen(XDPY)));
+        if (clipregion != None)
+            XftDrawSetClip(xft_alpha_draw, clipregion);
+        return xft_alpha_draw;
+      }
+
+    return 0;  //PENDING: warn? throw?
+}
+#endif
+
 - (void) copyBits: (XGGState*)source fromRect: (NSRect)aRect 
 				      toPoint: (NSPoint)aPoint
 {
@@ -464,29 +515,17 @@ static	Region	emptyRegion;
      
   // --- determine region to draw --------------------------------------
 
-  {
-    NSRect flushRect;                        // destination rectangle
-                                             // in View coordinates
+  srect = XGViewRectToX(source, fromRect);
+  toXPoint = XGViewPointToX(self, toPoint);
+  drect.x = toXPoint.x; 
+  drect.y = toXPoint.y - srect.height; 
+  drect.width = srect.width;
+  drect.height = srect.height;
 
-    flushRect.size = fromRect.size;
-    flushRect.origin = toPoint;
-    
-    flushRect.origin = [ctm pointInMatrixSpace: flushRect.origin];
-    drect = XGWindowRectToX(self, flushRect);
-    if (source->ctm->matrix.m22 < 0) drect.y += drect.height;
-
-    toXPoint.x = drect.x; 
-    toXPoint.y = drect.y; 
-
-    srect = XGViewRectToX (source, fromRect);
-
-    clipXRectsForCopying (source_win, &srect, dest_win, &drect);
-
-    if (XGIsEmptyRect(drect))
+  clipXRectsForCopying(source_win, &srect, dest_win, &drect);
+  if (XGIsEmptyRect(drect))
       return;
 
-  }
-
   // --- get destination XImage ----------------------------------------
   
   if (draw == dest_win->ident && dest_win->visibility < 0)
@@ -572,6 +611,7 @@ static	Region	emptyRegion;
                 fromRect: (NSRect)aRect
                  toPoint: (NSPoint)aPoint
                       op: (NSCompositingOperation)op
+		fraction: (float)delta
 {
   BOOL do_copy, source_alpha;
   XGCValues comp_gcv;
@@ -695,7 +735,7 @@ static	Region	emptyRegion;
             fromRect: aRect
             toPoint: aPoint
             op: op
-            fraction: 1];
+            fraction: delta];
     }
   
 
@@ -706,32 +746,28 @@ static	Region	emptyRegion;
     }
 }
 
+- (void) compositeGState: (GSGState *)source 
+                fromRect: (NSRect)aRect
+                 toPoint: (NSPoint)aPoint
+                      op: (NSCompositingOperation)op
+{
+  [self compositeGState: source 
+               fromRect: aRect
+                toPoint: aPoint
+                     op: op
+               fraction: 1.0];
+}
+
 - (void) dissolveGState: (GSGState *)source
 	       fromRect: (NSRect)aRect
 		toPoint: (NSPoint)aPoint 
 		  delta: (float)delta
 {
-  /* If we have no drawable, we can't proceed. */
-  if (draw == 0)
-    {
-      DPS_WARN(DPSinvalidid, @"No Drawable defined for dissolve");
-      return;
-    }
-
-  if (shouldDrawAlpha == NO)
-    {
-      /* No alpha buffers */
-      [self copyBits: (XGGState *)source fromRect: aRect toPoint: aPoint];
-    }
-  else
-    {
-      [self _compositeGState: (XGGState *)source 
-            fromRect: aRect
-            toPoint: aPoint
-            op: NSCompositeSourceOver
-            fraction: delta];
-    }
-
+  [self compositeGState: source 
+               fromRect: aRect
+                toPoint: aPoint
+                     op: NSCompositeSourceOver
+               fraction: delta];
 }
 
 - (void) compositerect: (NSRect)aRect
@@ -943,7 +979,7 @@ static	Region	emptyRegion;
 	    }
 	  y0 = pts[i].y * 2;
 	  y1 = p1.y * 2;
-	  if ((y0 < yh  &&  yh < y1) || (y1 < yh  &&  yh < y0) )
+	  if ((y0 < yh  &&  yh < y1) || (y1 < yh  &&  yh < y0))
 	    {
 	      int dy = yh - pts[i].y * 2;
 	      int ldy = y1 - y0;
@@ -1029,7 +1065,7 @@ static	Region	emptyRegion;
       BOOL doit;
       BOOL complex = NO;
       
-      for(j = 0; j < count; j++) 
+      for (j = 0; j < count; j++) 
         {
 	  doit = NO;
 	  type = [flatPath elementAtIndex: j associatedPoints: points];
@@ -1552,7 +1588,7 @@ static	Region	emptyRegion;
     bytesPerRow = (bitsPerPixel * pixelsWide) / 8;
 
   /* make sure its sane - also handles row padding if hint missing */
-  while((bytesPerRow * 8) < (bitsPerPixel * pixelsWide))
+  while ((bytesPerRow * 8) < (bitsPerPixel * pixelsWide))
     bytesPerRow++;
 
   /* get the colour space */
@@ -1561,12 +1597,12 @@ static	Region	emptyRegion;
       if ([colorSpaceName isEqualToString: NSDeviceRGBColorSpace] ||
 	  [colorSpaceName isEqualToString: NSCalibratedRGBColorSpace])
 	cspace = rgb_colorspace;
-      else if([colorSpaceName isEqualToString: NSDeviceCMYKColorSpace])
+      else if ([colorSpaceName isEqualToString: NSDeviceCMYKColorSpace])
 	cspace = cmyk_colorspace;
-      else if([colorSpaceName isEqualToString: NSDeviceWhiteColorSpace] ||
+      else if ([colorSpaceName isEqualToString: NSDeviceWhiteColorSpace] ||
 	      [colorSpaceName isEqualToString: NSCalibratedWhiteColorSpace])
 	cspace = gray_colorspace;
-      else if([colorSpaceName isEqualToString: NSDeviceBlackColorSpace] ||
+      else if ([colorSpaceName isEqualToString: NSDeviceBlackColorSpace] ||
 	      [colorSpaceName isEqualToString: NSCalibratedBlackColorSpace])
         {
 	  cspace = gray_colorspace;
@@ -1576,7 +1612,7 @@ static	Region	emptyRegion;
         {
 	  // if we dont recognise the name use RGB or greyscale as appropriate
 	  NSLog(@"XGContext (DPSImage): Unknown colour space %@", colorSpaceName);
-	  if(samplesPerPixel > 2)
+	  if (samplesPerPixel > 2)
 	    cspace = rgb_colorspace;
 	  else
 	    cspace = gray_colorspace;
