@@ -18,7 +18,7 @@
    You should have received a copy of the GNU General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 */
 
@@ -998,7 +998,7 @@ NSMutableDictionary	*pasteboards = nil;
       NSLog(@"Help - pasteboard server connection has died!\n");
       exit(EXIT_FAILURE);
     }
-  if ([connection isKindOf: [NSConnection class]])
+  if ([connection isKindOfClass: [NSConnection class]])
     {
       NSEnumerator    *e = [pasteboards objectEnumerator];
       PasteboardObject	  *o;
